//
// zi80.h -- Z-80, 8080 and 8085 emulator.
//

#include "../utiltype.h"

/* Return bits from Zi80::Setup().  Deprecated. */
#define MEM_CHG	(1)
#define IO_CHG	(2)
#define TRICKY	(4)	/* like ld a,r */
#define RHALT	(8)
#define ILL		(16)

struct Zi80Reg {
	u16 AF, BC, DE, HL, IX, IY, SP, PC, AFp, BCp, DEp, HLp;
	u8 I, R, IM, IFF1, IFF2;
};

class Zi80 {
public:
	Zi80();

	virtual u8		OpcodeFetch(u16 address);	// look at m_cycles for timing
	virtual u8		IntrFetch();
	virtual	u8		DataRead(u16 address);
	virtual	void	DataWrite(u16 address, u8 data);
	virtual u8		PortRead(u16 address);
	virtual void	PortWrite(u16 address, u8 data);

	// The bus operation monitoring code from trs80gp can be implemented
	// as a utility class inside the virtual methods above.

	// This needs to go away.
	bool	mode_1_IRQ();	// returns true if accepted

	void	PowerUp();
	i32		Step();		// Do a single instruction; returns deprecated status.
	void	SetINT(u8 level);

protected:
	Zi80Reg	m_reg;
	i32		m_cycles;
	i32		m_tcount;
	i32		m_rcounter;
	u8		m_INT;

private:

	void	bus_op(int type, u16 addr, i32 cycles, u8 data);
	u8		fetch_opcode();
	u8		fetch_n();
	u16		fetch_nn();
	u8		load8(u16 mem);
	u16		load16(u16 mem);
	void	store8(u16 mem, u8 val);
	void	store16(u16 mem, u16 val);

	u16		ixy(u16 *xy);

	void	assign_s(int val);
	void	assign_z(int val);
	void	assign_h(int val);
	void	assign_pv(int val);
	void	assign_n(int val);
	void	assign_c(int val);

	void	add(u8 *acc, int val, int cry, int docarry);
	void	sub(u8 *acc, int val, int cry, int docarry);
	void	cp(int val);

	void	andz(int val);
	void	orz(int val);
	void	xorz(int val);

	void	ex_memsp(u16 *r);

	void	add16(u16 *acc, int val);
	void	adc16(u16 *acc, int val);
	void	sbc16(u16 *acc, int val);

	void	daa();

	void	rs_flags(u8 val);

	void	rlca();
	void	rla();
	void	rrca();
	void	rra();

	void	rlc(u8 *acc);
	void	rl(u8 *acc);
	void	rrc(u8 *acc);
	void	rr(u8 *acc);
	void	sla(u8 *acc);
	void	sra(u8 *acc);
	void	srl(u8 *acc);

	void	bit(u8 val, int bit);
	void	set(u8 *val, int bit);
	void	res(u8 *val, int bit);

	void	pop(u16 *val);
	void	push(u16 val);

	void	jump(int cond);
	i32		branch(int cond);
	void	call(int cond);

	void	in(u8 port, u8 *acc, int doflags);
	void	out(u8 port, u8 val);

	int		main00(u8 op);
	int		main01(u8 op);
	int		main10(u8 op);
	int		main11(u8 op);

	int		cb();
	int		ed();
	int		ixiy(u16 *xy);

	int		cb00(u8 op);
	int		cb01(u8 op);
	int		cb10(u8 op);
	int		cb11(u8 op);
};
