//
// RunLoad.h - detect and run loading code
//

#include "../../lib/zi80/zi80.h"

class RunLoad : public Zi80
{
public:
	RunLoad();

	virtual u8 OpcodeFetch(u16 address);
	virtual u8 IntrFetch();
	virtual u8 DataRead(u16 address);
	virtual void DataWrite(u16 address, u8 data);
	virtual u8 PortRead(u16 address);
	virtual void PortWrite(u16 address, u8 data);

	bool Analyze();

	virtual bool LoadNext(bool setreg, bool isROM) = 0;
	virtual void LoadMemory(unsigned char *mem, unsigned char *loaded, int pc) = 0;

	virtual bool ReadCassetteHeader() = 0;
	virtual bool ReadCassetteByte(unsigned char *data) = 0;

	void SetStopLoad(bool stop);

	bool m_stopLoad;

	bool m_verbose;

protected:
	int WrittenSize(int *addr);
	void CheckForPartialCopy(int entry);
	void WarnPortChanges();
	void ClearPortChanges();

	u8 m_mem[65536];
	u8 m_write[65536];
	u8 m_initial_load[65536];

	u8 m_port[256];
	u8 m_portwritten[256];

	int m_newWriteCountdown;
	bool m_clearWriteOnWrite;
};
