//
// SuperLoad.cpp -- join RunLoad and TRS80Loader together.
//

static const int basicLoadAddress = 17385;  // Model III cassette Basic

#include <string.h>

#include "SuperLoad.h"

SuperLoad::SuperLoad()
{
	m_dump = false;
	m_dumpData = true;
	m_pc = -1;
}

void SuperLoad::SetDump(bool on)
{
	m_dump = on;
}

void SuperLoad::SetDumpData(bool on)
{
	m_dumpData = on;
}

bool SuperLoad::LoadFileData(const char *filename)
{
	m_load.LoadFileData(filename);

	if (GetError())
	{
		return false;
	}
	CheckWarning();

	return true;
}

bool SuperLoad::ReadCassetteHeader()
{
	// DetectMedia() too fragile.  To do the proper simulation we would keep going
	// until EOF or we find a cassette leader and sync byte.  But practically speaking
	// we'll have a header coming right up and it will be a full one.  So just skip
	// ahead a bit.

	m_load.SeekMediaByte(7);

	m_load.DetectMedia();
	if (GetError())
	{
		return false;
	}
	CheckWarning();

	switch (m_load.GetMediaType())
	{
	case TRS80Loader::cassette250:
	case TRS80Loader::cassette500:
	case TRS80Loader::cassette1500:
		return true;
	default:
		return false;
	}
}

bool SuperLoad::ReadCassetteByte(unsigned char *data)
{
	int byte = m_load.GetMediaByte(0);
	if (byte < 0)
	{
		return false;
	}
	*data = byte;
	m_load.SeekMediaByte(1);

	return true;
}

bool SuperLoad::LoadNext(bool setreg, bool isROM)
{
	m_load.DisposeFile();

	if (isROM)
	{
		m_load.LoadROM();
	}
	else
	{
		m_load.LoadRecords(basicLoadAddress);
	}

	if (m_dump)
	{
		m_load.DumpRecords(m_dumpData);
	}

	if (GetError())
	{
		return false;
	}
	CheckWarning();

	int pc = -1;
	int hl = -1;
	m_load.m_file->LoadAll(m_mem, &pc, &hl, m_write);

	if (setreg)
	{
		if (pc != -1)
		{
			m_reg.PC = m_pc;
		}
		if (hl != -1)
		{
			m_reg.HL = hl;
		}
	}

	if (pc != -1)
		m_pc = pc;

	LoadMemory(m_mem, m_write, m_pc);

	// XXX - but what of HL!

	return true;
}

void SuperLoad::LoadMemory(unsigned char *mem, unsigned char *loaded, int pc)
{
	m_load.LoadMemory(mem, loaded, pc);
}

char *SuperLoad::GetError()
{
	return m_load.GetErrorMessage();
}


void SuperLoad::CheckWarning()
{
	for (TRS80Record *rec = m_load.m_file; rec; rec = rec->m_next)
	{
		if (rec->m_action == TRS80Record::warning)
		{
			Warning((char *)rec->m_data);
		}
	}
}
