//
// SuperLoad.h -- join RunLoad and TRS80Loader together.
//
// General use is to translate format between .cas, .cmd, .hex, etc.  Can load
// more than one binary at once to concatenate them.
//
// Initially does not preserve the original structure and doesn't handle binary
// basic files in a sensible form.
//
// Suggestions: Control over BASIC load addresses.  Flag for low-speed cassette
// output. Tolerate checksum errors. Auto-load SYSTEM files.  Option to leave
// name out of .cmd output.
//
// Future possibilities include translations that preserve as much orignial
// structure as possible.  Simple dumps.  Module offsets.
//
// Look for '%d' formatting in output file name and copy out all the programs
// found in an input file.
//

#include <stdio.h>
#include <string.h>

#include "loader.h"
#include "RunLoad.h"

class SuperLoad : public RunLoad
{
public:
	SuperLoad();

	bool LoadFileData(const char *filename);

	TRS80Loader m_load;

	virtual bool LoadNext(bool setreg, bool isROM);
	virtual void LoadMemory(unsigned char *mem, unsigned char *loaded, int pc);
	virtual bool ReadCassetteHeader();
	virtual bool ReadCassetteByte(unsigned char *data);

	char *GetError();

	virtual void Warning(const char *message) = 0;

	void SetDump(bool on);
	void SetDumpData(bool on);

protected:
	void CheckWarning();

	bool m_dump;
	bool m_dumpData;
	int m_pc;
};
